/*
 *
 * demo for the pursuit navigator features in lgl
 *
 * Tthe IHM is like a game.
 *
 * by Cyril Robin - LAAS 2011
 *
 */


#include "lgl/GeoData.hpp"
#include "lgl/VisibilityMap.hpp"
#include "lgl/PursuitGraph.hpp"
#include "lgl/PursuitNavigator.hpp"
#include "lgl/LGLPursuitViewer.hpp"

#include "qdisplay/init.hpp"

#include <string>
#include <cstdlib>


using namespace jafar::lgl;
using namespace jafar::qdisplay;
using namespace std;

string robotVisibPath;
string robotAccessPath;
string targetAccessPath;
int xRobot, yRobot, xTarget,yTarget ;
unsigned int horizon ; // temporal horizon
unsigned int viewLength; 
unsigned int temporalTolerance; 
unsigned int riskTermMaxValue ;

void displayDemo(int argc, char *argv[])
{

	//-----------------------------------//
	//---- Pre-requisite
	PursuitNavigator navigator ( robotAccessPath, targetAccessPath) ;

	GeoData geodata(robotVisibPath) ;
	geodata.loadVisibility(robotVisibPath);

	VisibilityMap myVisibilityMap ( &geodata, viewLength, 4 ) ;	

	//PursuitGraph myPG (navigator, myVisibilityMap); 
	PursuitGraph myPG (navigator, myVisibilityMap, temporalTolerance, riskTermMaxValue); 

	//-----------------------------------//
	//---- Display and control
	
	LGLPursuitViewer *alfred = new LGLPursuitViewer(myPG);
	
		// Set data and parameters
		alfred->setRobotAccess ( robotAccessPath );
		alfred->setTargetAccess( targetAccessPath );
		alfred->setRobotVisib  ( robotVisibPath);

		alfred->setRobotPosition  ( xRobot, yRobot );
		alfred->setTargetPosition ( xTarget, yTarget );

		alfred->setHorizon( horizon );

		// Start the "game"
		alfred->start();

	
	//-----------------------------------//
	// A-Star
	// Compute a path from the robot position to the target position
	std::cout << "--- TEST de A-Star : " << endl;
	timeval t ;	double d ;
	gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 ;

		vector< Cell_ID> v ;
		bool b;
		b = navigator.navAStar(xRobot,yRobot,xTarget,yTarget,v);

	gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 - d ;
	std::cout << " FOUND ? " << b << " ( et v a pour taille : " << v.size() << " et a été calculé en " << d << " ms.)" << endl;

	int ysize = navigator.getysize();
	std::cout << " ysize = " << ysize <<std::endl;
	vector< Cell_ID>::reverse_iterator rit ;
	for( rit=v.rbegin() ; rit < v.rend(); ++rit )
	    cout << "(" << (*rit)/ysize <<","<< (*rit)%ysize <<") " ;
	
	std::cout << std::endl;
		
	//alfred->displayPath(v);

	//-----------------------------------//
	// Nav2Set
	// Compute a path from the robot position to a set of position close to the target
	std::cout << "--- TEST de nav2Set : " << endl;

		// current pos
		std::set<Cell_ID> currTargetPos ;
		currTargetPos.insert( (xTarget    ) * ysize + (yTarget    ) );

		//goals
		std::set<Cell_ID> goals ;
		navigator.findTargetAccessibility(currTargetPos, goals);

		// Navigation to the set of goal positions
		gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 ;
		
		//b = navigator.nav2Set(xRobot,yRobot,goals,v);
		b = navigator.nav2Set( xRobot*ysize + yRobot,goals,v);

		gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 - d ;
		std::cout << " FOUND ? " << b << " ( et v a pour taille : " << v.size() << " et a été calculé en " << d << " ms.)" << endl;

	std::cout << " ysize = " << ysize <<std::endl;
	for( rit=v.rbegin() ; rit < v.rend(); ++rit )
	    cout << "(" << (*rit)/ysize <<","<< (*rit)%ysize <<") " ;
	
	std::cout << std::endl;
		
		alfred->displayPath2Set(v, goals);
	

	//-----------------------------------//
	
	while(1)
		qtSleep(30000);
	
	cout << "#--- done." << endl;
	exit(1);

}

////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	if (argc > 11 )
	{
		robotAccessPath		= argv[1];
		targetAccessPath	= argv[2];
		robotVisibPath 		= argv[3];

		xRobot		= atoi(argv[4]) ; 	yRobot 		= atoi(argv[5]) ;
		xTarget		= atoi(argv[6]) ;	yTarget 	= atoi(argv[7]) ;

		horizon 			= atoi(argv[8]) ; 
		viewLength 			= atoi(argv[9]) ; 
		temporalTolerance 	= atoi(argv[10]) ; 
		riskTermMaxValue 	= atoi(argv[11]) ; 

		cout << "#[III] User parameter: robot accessibility filename \t= " << robotAccessPath << endl;
		cout << "#[III] User parameter: target accessibility filename \t= " << targetAccessPath << endl;
		cout << "#[III] User parameter: robot visibility filename \t= " << robotVisibPath << endl;
		
		cout << "#[III] User parameter: robot in ("<<xRobot<<","<<yRobot<<") and target in ("<<xTarget<<","<<yTarget<<")"<<endl;
		cout << "#[III] User parameter: horizon = "<< horizon << " and viewLength = " << viewLength << endl;
		cout << "#[III] User parameter: temporalTolerance  = " << temporalTolerance << " and riskTermMaxValue = " << riskTermMaxValue << endl;

	} else {

		cout << "Not enough arguments: "<< argv[0] << " <bitmap_robotaccessibilityfile> <bitmap_robotaccessibilityfile> <bitmap_visibilityfile> xRobot yRobot xTarget yTarget temporal_horizon viewLength temporalTolerance riskTermMaxValue" << endl;
		return 1;
	}

	QtAppStart((FUNC)displayDemo);
	
	return 0;
}

