/*
 *
 * demo for the pursuit navigator  and visibility features in lgl
 *
 * This file is made the Onera, and explains how to use primary functions of LGL
 * (path cost, visibility relation) as a library.
 * 
 * by Cyril Robin and Redouane Boughmar - LAAS 2012
 *
 */


#include "lgl/GeoData.hpp"
#include "lgl/VisibilityMap.hpp"
#include "lgl/PursuitNavigator.hpp"
#include "lgl/NavGraph.hpp"


#include <string>
#include <iostream>

using namespace jafar::lgl;
using namespace std;

string robotVisibPath;
string robotAccessPath;
string targetAccessPath;
int xRobot, yRobot, xTarget,yTarget ;
unsigned int viewLength; 

void displayDemo(int argc, char *argv[])
{

	//-----------------------------------//
	//------ Navigation
	// L'objet CHEMIN qui contiendra tous les waypoints
	NavPath superChemin;
	
	// Le point de depart et d'arrivé par exemple:
	NavLocation locStart( xRobot, yRobot);
	NavLocation locGoal ( xTarget, yTarget);
	
	// L'attribut de cout du chemin
	double coutDuChemin = 0.0;
	
	// Chargement du GeoData a partir d'une carte de traversabilite
	// greyscale ou RGB
	GeoData* geodata = new GeoData(robotAccessPath);
	
	// Creation de la decomposition necessaire à la construction du graphe de
	// navigation
	Quadtree<double, GeoData> *qt = new Quadtree<double, GeoData>(geodata);
	
	// Creation du graphe de navigation
	NavGraph *NG = new NavGraph(geodata, qt);
	
	// Executer une recherche de chemin
	NG->fastFindPath(locStart, locGoal, superChemin, coutDuChemin);
	
	// Et Voila !
	std::cout << " cout du chemin = " << coutDuChemin <<std::endl;
	cout << endl  ;

	//-----------------------------------//
	//------ Visibilite
	//Charger le fichier bitmap de visibilite
	GeoData geodata_Visu(robotVisibPath) ;							// Visibility data
	geodata_Visu.loadVisibility(robotVisibPath);

	VisibilityMap myVisibilityMap ( &geodata_Visu, viewLength ) ;	

	//-----------------------------------------//
	//---- Building the whole visibility Map
	cout << "#--- Building the whole visibility map." << endl;

	// NB: (pre)calcule toute la carte de visibilite en une fois. Peux prendre un certain temps.
	// D'autres options sont possibles (calcul a la demande) : voire la doc / le .hpp
	myVisibilityMap.computeWholeVisibility();

	cout << "Visibility map built." << endl << endl ;

	//-----------------------------------------//
	//---- Asking for visibility Area
	// Contient la liste des points visibles depuis (x,y)
	// Les points sont des Cell_ID (= x * ysize + y )
	cout << "#--- Accessing to a specific visible Area." << endl;

	VisibleArea myVisiblePoints ; 
	myVisiblePoints = myVisibilityMap.getVisibility( xRobot, yRobot)  ;

	// Affichage de ces points visibles
	int ysize = myVisiblePoints.getysize() ;			// taille de la carte, pour passer de "Cell_ID" aux cases
	std::cout << " ysize = " << ysize <<std::endl;

	set< Cell_ID>::iterator it, itb, ite ;
	itb = myVisiblePoints.visibleArea.begin() ; 
	ite = myVisiblePoints.visibleArea.end() ; 
	
	for ( it = itb ; it != ite ; ++ it)
	    cout << "(" <<  ( (*it) / ysize ) / 2  << "," << ( (*it) % ysize ) / 2 << ") " ; 	// factor 2 between cell/positions and node/pixel

	std::cout << std::endl;

	//-----------------------------------//
	cout << "#--- done." << endl;
	return;

}

////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	if (argc > 7 )
	{
		robotAccessPath		= argv[1];
		robotVisibPath 		= argv[2];

		xRobot		= atoi(argv[3]) ; 	
		yRobot 		= atoi(argv[4]) ;
		xTarget		= atoi(argv[5]) ;	
		yTarget 	= atoi(argv[6]) ;

		viewLength 			= atoi(argv[7]) ; 

		cout << "#[III] User parameter: robot accessibility filename \t= " << robotAccessPath << endl;
		cout << "#[III] User parameter: robot visibility filename \t= " << robotVisibPath << endl;
		
		cout << "#[III] User parameter: robot in ("<<xRobot<<","<<yRobot<<") and target in ("<<xTarget<<","<<yTarget<<")"<<endl;
		cout << "#[III] User parameter: viewLength = " << viewLength << endl;

		cout << endl  ;

	} else {

		cout << "Not enough arguments: "<< argv[0] << " <bitmap_robotaccessibilityfile> <bitmap_visibilityfile> xRobot yRobot xTarget yTarget viewLength " << endl;
		return 1;
	}

	displayDemo(argc, argv);
	
	return 0;
}

