/*
 *
 * demo for the Pursuit features in lgl
 *
 * The IHM is like a game.
 *
 * by Cyril Robin - LAAS 2011
 *
 */


#include "lgl/GeoData.hpp"
#include "lgl/VisibilityMap.hpp"
#include "lgl/PursuitGraph.hpp"
#include "lgl/PursuitNavigator.hpp"
#include "lgl/LGLPursuitViewer.hpp"

#include "qdisplay/init.hpp"

#include <string>


using namespace jafar::lgl;
using namespace jafar::qdisplay;
using namespace std;

string robotVisibPath;
string robotAccessPath;
string targetAccessPath;
int xRobot, yRobot, xTarget,yTarget ;
unsigned int horizon ; // temporal horizon
unsigned int viewLength; // viewLength
unsigned int temporalTolerance; 
unsigned int riskTermMaxValue ;

void displayDemo(int argc, char *argv[])
{

	//-----------------------------------//
	//---- Pre-requisite
	PursuitNavigator navigator ( robotAccessPath, targetAccessPath) ; 	// Navigation data

	GeoData geodata(robotVisibPath) ;									// Visibility data
	geodata.loadVisibility(robotVisibPath);

	VisibilityMap myVisibilityMap ( &geodata, viewLength, 4 ) ;	

	//PursuitGraph myPG (navigator, myVisibilityMap); 
	PursuitGraph myPG (navigator, myVisibilityMap, temporalTolerance, riskTermMaxValue); 

	//-----------------------------------//
	//---- Display and control
	
	LGLPursuitViewer *alfred = new LGLPursuitViewer(myPG);
	
		// Set data and parameters
		alfred->setRobotAccess ( robotAccessPath );
		alfred->setTargetAccess( targetAccessPath );
		alfred->setRobotVisib  ( robotVisibPath);

		alfred->setRobotPosition  ( xRobot, yRobot );
		alfred->setTargetPosition ( xTarget, yTarget );

		alfred->setHorizon( horizon );

		// Start the "game"
		alfred->start();
	
	//-----------------------------------//
	//-----------------------------------//
	
	while(1)
		qtSleep(30000);

	cout << "#--- done." << endl;

}

////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	if (argc > 11 )
	{
		robotAccessPath		= argv[1];
		targetAccessPath	= argv[2];
		robotVisibPath 		= argv[3];

		xRobot		= atoi(argv[4]) ; 	yRobot 		= atoi(argv[5]) ;
		xTarget		= atoi(argv[6]) ;	yTarget 	= atoi(argv[7]) ;

		horizon 			= atoi(argv[8]) ; 
		viewLength 			= atoi(argv[9]) ; 
		temporalTolerance 	= atoi(argv[10]) ; 
		riskTermMaxValue 	= atoi(argv[11]) ; 

		cout << "#[III] User parameter: robot accessibility filename \t= " << robotAccessPath << endl;
		cout << "#[III] User parameter: target accessibility filename \t= " << targetAccessPath << endl;
		cout << "#[III] User parameter: robot visibility filename \t= " << robotVisibPath << endl;
		
		cout << "#[III] User parameter: robot in ("<<xRobot<<","<<yRobot<<") and target in ("<<xTarget<<","<<yTarget<<")"<<endl;
		cout << "#[III] User parameter: horizon = "<< horizon << " and viewLength = " << viewLength << endl;
		cout << "#[III] User parameter: temporalTolerance  = " << temporalTolerance << " and riskTermMaxValue = " << riskTermMaxValue << endl;

		cout << std::endl ;

	} else {

		cout << "Not enough arguments: "<< argv[0] << " <bitmap_robotaccessibilityfile> <bitmap_targetaccessibilityfile> <bitmap_visibilityfile> xRobot yRobot xTarget yTarget temporal_horizon viewLength temporalTolerance riskTermMaxValue" << endl;
		return 1;
	}

	QtAppStart((FUNC)displayDemo);
	
	return 0;
}

