/*
 *
 * demo file for the Pursuit features in lgl
 *
 * by Cyril Robin - LAAS 2011
 *
 */


#include "qdisplay/init.hpp"

#include "lgl/GeoData.hpp"
#include "lgl/LGLViewer.hpp"
#include "lgl/VisibilityMap.hpp"
#include "lgl/PursuitGraph.hpp"

#include <string>

using namespace jafar::lgl;
using namespace jafar::qdisplay;
using namespace std;

string robotVisibPath;
string robotAccessPath;
string targetAccessPath;
int xRobot, yRobot, xTarget,yTarget ;
unsigned int horizon ; // temporal horizon
unsigned int viewLength; // viewLength


void displayDemo(int argc, char *argv[])
{

	//timer
	timeval t ;
	double d ;

	//-----------------------------------------//
	//----- Loading the map
	cout << "#--- Loading the map." << endl;
		PursuitNavigator navigator ( robotAccessPath, targetAccessPath) ;

	//---- Loading the visibility file
	cout << "#--- Loading the visibility file." << endl;
		GeoData geodata(robotVisibPath) ;
		geodata.loadVisibility(robotVisibPath);

	//---- Building the visibility Map
	cout << "#--- Building the visibility map." << endl;
		VisibilityMap myVisibilityMap ( &geodata, viewLength, 4 ) ;	

	cout << endl;


	//-----------------------------------------//
	//---- Building the Pursuit Graph
	cout << "#--- Building Pursuit Graph" << endl;
		gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 ;

		PursuitGraph myPG ( navigator, myVisibilityMap ); 

		gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 - d ;
		cout << "#---III---  " << " Elapsed time : "<< d <<" milliseconds." << endl;


	//---- Computing the Pursuit Graph
	cout << "#--- Computing Pursuit Graph" << endl;
		gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 ;

		myPG.computeAll( xRobot, yRobot, xTarget, yTarget, horizon ) ; 			// Step constraint function
		//myPG.computeAllBefore( xRobot, yRobot, xTarget, yTarget, horizon*1000.0 ) ; 	// Time constraint function (in ms)

		gettimeofday(&t, NULL);	d = t.tv_sec*1000.0 + t.tv_usec/1000.0 - d ;
		cout << "#---III---  " << " Elapsed time : "<< d <<" milliseconds for horizon = " << myPG.gethorizon() << endl;

	cout << endl;
	

	//-----------------------------------------//
	//---- Updating the Pursuit Graph
	cout << "#--- Updating Pursuit Graph" << endl;

		//Cell_ID newRobotPos ;
		int xR, yR ;
		gettimeofday(&t, NULL);	d = t.tv_sec*1000.0 + t.tv_usec/1000.0 ;

		myPG.updatePG ( (xTarget + 2), (yTarget - 2), xR, yR); // NB: ( +2 , -2 ) = top-right movement 

		gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 - d ;
		cout << "#---III---  " 	<< " Elapsed time : "<< d <<" milliseconds with a new horizon of " << myPG.gethorizon() << endl;
		cout << "#---III---  "	<< " The new position of the robot is (" << xR << "," << yR 
								<< "), and the new front size is : " << myPG.getfront().size()<<  endl ;

	//---- completing the Pursuit Graph to the temporal horizon
	cout << "#--- Completing the Pursuit Graph until the temporal horizon" << endl;
		gettimeofday(&t, NULL);	d = t.tv_sec*1000.0 + t.tv_usec/1000.0 ;

		myPG.completeToHorizon(); 

		gettimeofday(&t, NULL) ; d = t.tv_sec*1000.0 + t.tv_usec/1000.0 - d ;
		cout << "#---III---  " << " Elapsed time : "<< d <<" milliseconds with a new horizon of " << myPG.gethorizon() << endl;

	cout << endl ;



	//-----------------------------------------//
	//---- Display
	cout << "#--- Building The Viewer." << endl;
		LGLViewer *albert = new LGLViewer(&myVisibilityMap, NULL, NULL, 1);

		//---- image
		albert->setImage(robotVisibPath, geodata.geoxroot(), geodata.geoyroot());

		//---- data
		//albert->displayPGFront( myPG );
		//albert->displayPGRisk ( myPG );
		albert->displayPGraph ( myPG, true );


	//-----------------------------------------//
	cout << "#--- done." << endl;
}


////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	if (argc > 9)
	{
		robotAccessPath		= argv[1];
		targetAccessPath	= argv[2];
		robotVisibPath 		= argv[3];

		xRobot		= atoi(argv[4]) ; 	yRobot 		= atoi(argv[5]) ;
		xTarget		= atoi(argv[6]) ;	yTarget 	= atoi(argv[7]) ;

		horizon 	= atoi(argv[8]) ; 
		viewLength 	= atoi(argv[9]) ; 

		cout << "#[III] User parameter: robot accessibility filename \t= " << robotAccessPath << endl;
		cout << "#[III] User parameter: target accessibility filename \t= " << targetAccessPath << endl;
		cout << "#[III] User parameter: robot visibility filename \t= " << robotVisibPath << endl;

	} else {

		cout << "Not enough arguments: "<< argv[0] << " <bitmap_robotaccessibilityfile> <bitmap_targetaccessibilityfile> <bitmap_visibilityfile> xRobot yRobot xTarget yTarget temporal_horizon viewLength" << endl;
		return 1;
	}

	QtAppStart((FUNC)displayDemo);
	
	return 0;
}

