/* $Id: test_suite_Quadtree.hpp 4272 2009-11-17 16:07:20Z rboumgha $ */

#include <time.h>
// test suite specific includes
#include "lgl/Quadtree.hpp"
#include "lgl/Decomp.hpp"
#include "lgl/GeoData.hpp"
#include "lgl/Raster.hpp"
#include "lgl/PositionManager.hpp"
#include "lgl/CostManager.hpp"
#include "lgl/NavGraph.hpp"
#include "lgl/LGLViewer.hpp"
#include "lgl/NavHeuDistances.hpp"
#include "qdisplay/init.hpp"
#include <string>


using namespace jafar::lgl;
using namespace jafar::qdisplay;
using namespace std;

string mapPath;

void displayDemo(int argc, char *argv[])
{
	// Time watfhers
	timeval tim;
	double t1, t2;
	// Path iterator
	typedef NavPath::const_iterator navpath_iterator;
	NavPath navPath;
	// Positions
	NavLocation locStart(0, 0); 
	NavLocation locGoal(400, 400); 

	//----- Loading the map
	cout << "#--- Loading the map." << endl;
	GeoData geodata(mapPath);
	// Decomposition type: HT_NONE, SIMPLE, NEARLY, SIGMA
	geodata.setDecompositionType(HT_NONE);

	//----- Building the map decomposition using a QuadTree
	cout << "#--- Building the Quadtree." << endl;
	Quadtree<double, GeoData> qt(&geodata);
	// Textual output (gnuplot)
	//cout << *qt;

	// Get the last quadtree decomposition to display it.
	cout << "#--- Extracting Decomposition." << endl;
	Decomp* decomp = qt.getDecomp();
	// Textual output (gnuplot)
	//cout << *decomp;

	//------ Construct a Nav Graph
	cout << "#--- Building Navigation Graph: ";
	gettimeofday(&tim, NULL);
	t1=tim.tv_sec+(tim.tv_usec/1000000.0);
		NavGraph NG(&geodata, &qt);
	gettimeofday(&tim, NULL);
	t2=tim.tv_sec+(tim.tv_usec/1000000.0);
	cout << (t2-t1) << "s." << endl;

	//----- Compute the path from Start to Goal using A*
	cout << "#--- Looking for a path: ";
	gettimeofday(&tim, NULL);
	t1=tim.tv_sec+(tim.tv_usec/1000000.0);
		NG.findPath(locStart, locGoal, navPath);
	gettimeofday(&tim, NULL);
	t2=tim.tv_sec+(tim.tv_usec/1000000.0);
	cout << (t2-t1) << "s." << endl;

	cout << "#--- Looking for a path in a fast way: ";
	gettimeofday(&tim, NULL);
	t1=tim.tv_sec+(tim.tv_usec/1000000.0);
		NG.fastFindPath(locStart, locGoal, navPath);
	gettimeofday(&tim, NULL);
	t2=tim.tv_sec+(tim.tv_usec/1000000.0);
	cout << (t2-t1) << "s." << endl;

	//-----------------------------------------//
	// Display
	cout << "#--- Building The Viewer." << endl;
	LGLViewer *robert = new LGLViewer(decomp, NG.getGraph(), 1);
	cout << "#--- Viewing the Decomposition." << endl;
	//-----------------------------------------//
	robert->setImage(mapPath, geodata.geoxroot(), geodata.geoyroot());
	//-----------------------------------------//
	robert->displayDecomp();
	//-----------------------------------------//
	robert->displayGraph(true, true);
	//-----------------------------------------//
	robert->displayPath(navPath);
	//-----------------------------------------//
	
	cout << "#--- Saving Images " << endl;
	geodata.saveTraversability("/tmp/r_traversabilityMap.ppm");

	//-----------------------------------------//
	cout << "#---III---  " << NG.getNumberOfVertices()  << " number of vertices." << endl;
	cout << "#---III---  " << NG.getNumberOfEdges()     << " number of edges." << endl;
	cout << "#---III---  " << NG.getNumberOfEdges()/2   << " vertices' connexions." << endl;
	cout << "#--- done." << endl;
}

////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	mapPath = "";
	if (argc > 1)
    {
        mapPath = argv[1];
        cout << "#[III] User parameter: map filename = " << mapPath << endl;
    } else {
		cout << "Not a good number of arguments: "<< argv[0] << "  <bitmap_mapfile>" << endl;
        return 1;
    }

	QtAppStart((FUNC)displayDemo);
	return 0;
}

