/*
 *
 * demo for the path planning system in lgl
 *
 * by Redouane Boughmar and Cyril Robin - LAAS 2011
 *
 */


#include <time.h>
// test suite specific includes
#include "lgl/Quadtree.hpp"
#include "lgl/Decomp.hpp"
#include "lgl/GeoData.hpp"
#include "lgl/Raster.hpp"
#include "lgl/PositionManager.hpp"
#include "lgl/CostManager.hpp"
#include "lgl/NavGraph.hpp"
#include "lgl/LGLViewer.hpp"
#include "lgl/NavHeuDistances.hpp"
#include "qdisplay/init.hpp"
#include <string>


using namespace jafar::lgl;
using namespace jafar::qdisplay;
using namespace std;

string mapPath;
int xStart, yStart ;
int xGoal, yGoal ; 

void displayDemo(int argc, char *argv[])
{
	// Time watfhers
	timeval t;
	double d ;
	// Path iterator
	typedef NavPath::const_iterator navpath_iterator;
	NavPath navPath;
	// Positions
	NavLocation locStart(xStart, yStart); 
	NavLocation locGoal(xGoal, yGoal); 


	//----- Loading the map
	cout << "#--- Loading the map." << endl;
	GeoData geodata(mapPath);
	// Decomposition type: HT_NONE, SIMPLE, NEARLY, SIGMA
	geodata.setDecompositionType(HT_NONE);

	//----- Building the map decomposition using a QuadTree
	cout << "#--- Building the Quadtree." << endl;
	Quadtree<double, GeoData> qt(&geodata);
	// Textual output (gnuplot)
	//cout << *qt;

	// Get the last quadtree decomposition to display it.
	cout << "#--- Extracting Decomposition." << endl;
	Decomp* decomp = qt.getDecomp();
	// Textual output (gnuplot)
	//cout << *decomp;

	//------ Construct a Nav Graph
	cout << "#--- Building Navigation Graph: ";
		gettimeofday(&t, NULL); d = t.tv_sec * 1000.0 + t.tv_usec / 1000.0 ;

		NavGraph NG(&geodata, &qt);

		gettimeofday(&t, NULL); d = t.tv_sec * 1000.0 + t.tv_usec / 1000.0 - d ;
		cout << "#---III--- Computed in " << d <<" ms." << endl;

	//----- Compute the path from Start to Goal using A*
	cout << "#--- Looking for a path: ";
		gettimeofday(&t, NULL); d = t.tv_sec * 1000.0 + t.tv_usec / 1000.0 ;

		NG.findPath(locStart, locGoal, navPath);

		gettimeofday(&t, NULL); d = t.tv_sec * 1000.0 + t.tv_usec / 1000.0 - d ;
		cout << "#---III--- Computed in " << d <<" ms." << endl;

	cout << "#--- Looking for a path in a fast way: ";
		gettimeofday(&t, NULL); d = t.tv_sec * 1000.0 + t.tv_usec / 1000.0 ;

		NG.fastFindPath(locStart, locGoal, navPath);

		gettimeofday(&t, NULL); d = t.tv_sec * 1000.0 + t.tv_usec / 1000.0 - d ;
		cout << "#---III--- Computed in " << d <<" ms." << endl;

	//-----------------------------------------//
	// Display
	cout << "#--- Building The Viewer." << endl;
	LGLViewer *robert = new LGLViewer(decomp, NG.getGraph(), 1);
	cout << "#--- Viewing the Decomposition." << endl;
	//-----------------------------------------//
	robert->setImage(mapPath, geodata.geoxroot(), geodata.geoyroot());
	//-----------------------------------------//
	robert->displayDecomp();
	//-----------------------------------------//
	robert->displayGraph(true, true);
	//-----------------------------------------//
	robert->displayPath(navPath);
	//-----------------------------------------//
	
	cout << "#--- Saving Images " << endl;
	geodata.saveTraversability("/tmp/r_traversabilityMap.ppm");

	//-----------------------------------------//
	cout << "#---III---  " << NG.getNumberOfVertices()  << " number of vertices." << endl;
	cout << "#---III---  " << NG.getNumberOfEdges()     << " number of edges." << endl;
	cout << "#---III---  " << NG.getNumberOfEdges()/2   << " vertices' connexions." << endl;

	cout << "#--- done." << endl;
}

////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	mapPath = "";
	if (argc > 5)
    {
        mapPath = argv[1];
        xStart	= atoi(argv[2]); 	yStart	= atoi(argv[3]);
        xGoal	= atoi(argv[4]);	yGoal	= atoi(argv[5]);

        cout << "#[III] User parameter: map filename = " << mapPath << endl;

    } else {
		cout << "Not a good number of arguments: "<< argv[0] << "  <bitmap_mapfile> xStart yStart xGoal yGoal" << endl;
        return 1;
    }

	QtAppStart((FUNC)displayDemo);
	return 0;
}

