/*
 *
 * demo for the visibility features in lgl
 *
 * by Cyril Robin - LAAS 2011
 *
 */


#include "lgl/Quadtree.hpp"
#include "lgl/Decomp.hpp"
#include "lgl/GeoData.hpp"
#include "lgl/Raster.hpp"
#include "lgl/PositionManager.hpp"
#include "lgl/CostManager.hpp"
#include "lgl/NavGraph.hpp"
#include "lgl/LGLViewer.hpp"
#include "lgl/NavHeuDistances.hpp"
#include "lgl/VisibleArea.hpp"
#include "lgl/VisibilityMap.hpp"
#include "lgl/NavGNode.hpp"
#include "qdisplay/init.hpp"
#include <string>

using namespace jafar::lgl;
using namespace jafar::qdisplay;
using namespace std;

string visibilityPath;
int xGoal,yGoal ;
unsigned int viewLength;


void displayDemo(int argc, char *argv[])
{

	//----- Loading the map
	cout << "#--- Loading the map." << endl;
	GeoData geodata(visibilityPath);
	//// Decomposition type: HT_NONE, SIMPLE, NEARLY, SIGMA
	geodata.setDecompositionType(HT_NONE);

	////----- Building the map decomposition using a QuadTree
	cout << "#--- Building the Quadtree." << endl;
	Quadtree<double, GeoData> qt(&geodata);

	//---- Loading the visibility file
	cout << "#--- Loading the visibility file." << endl;
	geodata.loadVisibility(visibilityPath);

	cout << endl ;

	//-----------------------------------------//
	//---- Building the whole visibility Map
	cout << "#--- Building the whole visibility map." << endl;
		timeval t ; double d ;
		gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 ;

		VisibilityMap myVisibilityMap ( &geodata, viewLength, 1 ) ;
		//myVisibilityMap.computeWholeVisibility();

		gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 - d ;
		cout << "#---III---  " << "Elapsed time : "<< d <<" milliseconds." << endl;


	//---- Building a partial visibility Map
	cout << "#--- Building a partial visibility map." << endl;

		std::map<Cell_ID,VisibleArea> myPartialMap ;
		std::set<Cell_ID> mySet ;

		mySet.insert( (Cell_ID) 0   );
		mySet.insert( (Cell_ID) 91  );
		mySet.insert( (Cell_ID) ( xGoal * myVisibilityMap.getysize() + yGoal ) );

		gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 ;

		myVisibilityMap.computePartialVisibility( myPartialMap, mySet);	

		gettimeofday(&t, NULL); d = t.tv_sec*1000.0 + t.tv_usec/1000.0 - d ;
		cout << "#---III---  " << "Elapsed time : "<< d <<" milliseconds for " << myPartialMap.size() << " nodes." << endl;

	cout << endl ;


	//-----------------------------------------//
	// Clearing the VisibilityMap
	//myVisibilityMap.clearVisibility();
	

	//-----------------------------------------//
	//---- Accessing data
	cout << "******* Access to visibility band : " ;
	cout << (double) geodata.getValueFromBand(GEODATA_VISIBILITY_BAND, (xGoal-1)/2, (yGoal-1)/2 ) << endl;

	cout << "******* Access to visibility height band : " ;
	cout << (double) geodata.getValueFromBand(GEODATA_VISIBILITY_HEIGHT_BAND, (xGoal-1)/2, (yGoal-1)/2 ) << endl;

	//cout << "******* Printing a visible Area in text mode." << endl;
	//NB : output in texte mode 
	//myVisibilityMap.getVisibility(xGoal,yGoal).printVisibleArea();

	cout << endl ;


	//-----------------------------------------//
	// Display
	cout << "#--- Building The Viewer." << endl;
	LGLViewer *albert = new LGLViewer(&myVisibilityMap, NULL, NULL, 1);

	albert->setImage(visibilityPath, geodata.geoxroot(), geodata.geoyroot());

	//albert->displayVisibilityMap(xGoal, yGoal, false, true, false);
	//albert->displayVisibilityMap( 0, 0, false, true, false);
	
	std::map<Cell_ID, VisibleArea>::iterator itPM;
	itPM = myPartialMap.find( (Cell_ID) ( xGoal * myVisibilityMap.getysize() + yGoal ) );
	albert->displayVisibleArea( (*itPM).second, true, true, false );


	//-----------------------------------------//
	cout << "#--- done." << endl;
}

////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	if (argc > 4)
	{
		visibilityPath = argv[1];
		xGoal= atoi(argv[2]) ; yGoal = atoi(argv[3]) ;
		viewLength = atoi(argv[4]);
		cout << "#[III] User parameter: visibility filename = " << visibilityPath << endl;
		cout << "#[III] User parameter: Goal =  ( " << xGoal << " , " << yGoal  << " )"<< endl;
		cout << "#[III] User parameter: viewLength =  " << viewLength << endl;
		cout << endl ;

	} else {
		cout << "Not enough arguments: "<< argv[0] << " <bitmap_visibilityfile> xGoal yGoal viewLength" << endl;
		return 1;

	}

	QtAppStart((FUNC)displayDemo);
	return 0;
}

