/* $Id: lgl.i 4550 2010-06-09 12:49:17Z rboumgha $ */

/** swig interface file for module lgl.
 *
 * \file lgl.i
 * \ingroup lgl
 */      

%module lgl


%{

/* ruby defines ALLOC which conflicts with boost */ 
#undef ALLOC
/* More ignore for lgl symbol */
#undef T_SYMBOL
#undef T_CLASS

  
/* 
 * headers necessary to compile the wrapper
 */

#include "jafarConfig.h"

#include <iostream>

#include <list>
#include <vector>

#include "lgl/Raster.hpp"
#include "lgl/RasterBand.hpp"
#include "lgl/InfoBand.hpp"
#include "lgl/LandmarkBand.hpp"

#include "lgl/GeoData.hpp"
#include "lgl/Location.hpp"

#include "lgl/Decomp.hpp"
#include "lgl/Decomposer.hpp"
#include "lgl/Quadtree.hpp"

#include "lgl/PositionManager.hpp"

#include "lgl/GraphPath.hpp"
#include "lgl/Graphs.hpp"
#include "lgl/NavGraph.hpp"

#include "lgl/CostManager.hpp"

#include "lgl/LGLViewer.hpp"

using namespace jafar::lgl;

%}

%include "jafar.i"
%include "lglException.i"

%include "std_vector.i"

// treat some references and pointers 
// use : i = Jafar::Lgl::new_intp()
// vi = jafar::Lgl::floatp_value()
%include "cpointer.i"
%pointer_functions(int, intp);
%pointer_functions(float, floatp);

/*
 * headers to be wrapped goes here
 */

// %include "lglTools.i"
// instantiate some print functions
// replace "Type" with appropriate class name
// %template(print) jafar::lgl::print<jafar::lgl::Type>;

// %include "lglException.i"
%include "lglTools.i"

%ignore operator<<;
%ignore operator!=;
%ignore operator=;
%ignore operator++;
%ignore operator==;

%include "lgl/Raster.hpp"
%include "lgl/Location.hpp"
%include "lgl/GeoData.hpp"

%include "lgl/RasterBand.hpp"
%template(GDRasterBandF)     jafar::lgl::GDRasterBand<float>;
%template(GDRasterBandD)     jafar::lgl::GDRasterBand<double>;
%include "lgl/InfoBand.hpp"
%include "lgl/LandmarkBand.hpp"

%include "lgl/PositionManager.hpp"

%include "lgl/Decomp.hpp"
%template(LocationVector)  std::vector<jafar::lgl::NavLocation>;
%template(CellVector)  std::vector<jafar::lgl::RasterCellIndex>;
 
%include "lgl/Decomposer.hpp"
%template(TDecomposerF)      jafar::lgl::TDecomposer<float,jafar::lgl::GeoData>;
%template(TDecomposerD)      jafar::lgl::TDecomposer<double,jafar::lgl::GeoData>;

%include "lgl/Quadtree.hpp"
%template(QuadtreeF)         jafar::lgl::Quadtree<float,jafar::lgl::GeoData>;
%template(QuadNodeF)     jafar::lgl::QuadNode<float,jafar::lgl::GeoData>;
%template(QuadNodeInternalF) jafar::lgl::QuadNodeInternal<float,jafar::lgl::GeoData>;
%template(QuadNodeLeafF)     jafar::lgl::QuadNodeLeaf<float,jafar::lgl::GeoData>;

%template(QuadtreeD)         jafar::lgl::Quadtree<double,jafar::lgl::GeoData>;
%template(QuadNodeD)     jafar::lgl::QuadNode<double,jafar::lgl::GeoData>;
%template(QuadNodeInternalD) jafar::lgl::QuadNodeInternal<double,jafar::lgl::GeoData>;
%template(QuadNodeLeafD)     jafar::lgl::QuadNodeLeaf<double,jafar::lgl::GeoData>;

namespace boost {}

%include "lgl/GraphPath.hpp"
%include "lgl/NavGEdge.hpp"
%include "lgl/NavGNode.hpp"
%include "lgl/Graphs.hpp"
%template(NavGraphT)  jafar::lgl::NavGraphT<jafar::lgl::NavGEdge,jafar::lgl::NavGNode,float>;
%template(NavGraphD)  jafar::lgl::NavGraphT<jafar::lgl::NavGEdge,jafar::lgl::NavGNode,double>;
%include "lgl/NavGraph.hpp"

// this wrapping should preceed the Nav because the latter uses this class
%include "lgl/CostManager.hpp"

%template(Path)  jafar::lgl::GraphPath<jafar::lgl::NavLocation>;

%include "lgl/LGLViewer.hpp"

