/**
 * \file LGLPursuitViewer.hpp
 *
 * Header file to declare Qt drawing objects and functions 
 * Used as a small IHM for the pursuit Module.
 *
 * \author Cyril Robin
 *
 * \ingroup lgl
 *
 **/

/* $Id: lglViewer.hpp 4094 2009-09-04 09:32:58Z hcao $ */

#ifdef HAVE_MODULE_QDISPLAY // lgl viewer is on if HAVE_MODULE_QDISPLAY is defined

#ifndef LGL_DISPLAY_MULTI_PURSUIT_HPP
#define LGL_DISPLAY_MULTI_PURSUIT_HPP

#include <qdisplay/init.hpp>
#include <qdisplay/Viewer.hpp>
#include <qdisplay/ImageView.hpp>
#include <qdisplay/Shape.hpp>

#include <lgl/PursuitStats.hpp>
#include <lgl/RobotPursuer.hpp>

#include <boost/thread/thread.hpp>

#include <ostream>
#include <string>
#include <vector>

namespace jafar {
	namespace lgl {


		class LGLMultiPursuit_Viewer ;

		/*  specific viewer, herited from qdisplay */
		class MultiPursuit_Controller : public jafar::qdisplay::Viewer 
		{///{{{
			
			private :
				LGLMultiPursuit_Viewer *master ;
			
			protected : 
				void setMaster( LGLMultiPursuit_Viewer &_master);

			signals:
				void keyPressEvent(QKeyEvent *event);

			friend class LGLMultiPursuit_Viewer ;

		};///}}}

		class MultiPursuit_Viewer : public jafar::qdisplay::Viewer 
		{///{{{

			private :
				double scale ;
			
			public :
				MultiPursuit_Viewer(double _scale) :
					scale(_scale)
				{ }
			
			protected :
				void mouseMoveEvent ( QMouseEvent * event );

			friend class LGLMultiPursuit_Viewer ;

		};///}}}

		struct MultiProtagonist
		{///{{{
			//position
			int x ;
			int y ;

			//display
			jafar::qdisplay::Shape *shape1;
			jafar::qdisplay::Shape *shape2;
			jafar::qdisplay::Shape *shape3;
		};///}}}

		class LGLMultiPursuit_Viewer 
		{///{{{
			public:
				
				// Constructors and Destructor 
				LGLMultiPursuit_Viewer( RobotPursuer &_pt_robot);
				
				~LGLMultiPursuit_Viewer();

				void setRobotPosition	( int _x, int _y);
				void setTargetPosition	( int _x, int _y);

				/* start the game 
				 *
				 * return false if encounter any problem while launching the game.
				 **/
				bool start () ;

				/* Reaction to target Movement (+ _x, + _y )
				 */
				void move ( int _x, int _y) ;

			private:
				/* robot */
				RobotPursuer *pt_robot;

				/* Controler */
				jafar::lgl::MultiPursuit_Controller* ctrl ;

				/** QDisplay viewer used to draw shapes */
				MultiPursuit_Viewer* viewer1;
				MultiPursuit_Viewer* viewer2;
				MultiPursuit_Viewer* viewer3;

				/* Position of the robot and the target */
				MultiProtagonist robot ;
				MultiProtagonist target ;

				/* Scale for display */
				double scale ;

				/* risk view */
				std::vector< jafar::qdisplay::Shape *> risks ;

				/* enable / disable the display of the risks
				 */
				bool visibleRisks;

				/* statistics 
				 */
				PursuitStats stats ;

				/* start the display 
				 *
				 * return false if data (file path) are missing, true otherwise.
				 **/
				bool startDisplay () ;

				/* Refresh the display (especially the protagonist position )
				 */
				bool refresh();

				/* Display the risk positions
				 */
				void displayrisks();

				/* Hide the risk positions
				 */
				void hiderisks();

				/* Display the statistics
				 */
				void displaystats();

				/* Display the "help"
				 */
				void displayhelp();

			friend class MultiPursuit_Controller ;

		};///}}}

	}
}

#endif // LGL_DISPLAY_MULTI_PURSUIT_HPP
#endif // QDisplay enabled 
