/**
 * \file NavGEdge.hpp
 *
 * Header file for the navigation graph edge structure
 *
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_NAVIGATION_GRAPH_EDGE_HPP
#define LGL_NAVIGATION_GRAPH_EDGE_HPP

#include <climits>

#include <lgl/Raster.hpp>

namespace jafar {
	namespace lgl {
		
		/**
		 * This structure models the navigation graph edge.
		 * This is a bundle for BGL graph properties on edges.
		 *
		 * Each edge has a double weight value composed by its other parameters
		 * thanks to the CostManager.
		 */
		struct NavGEdge {
			
			/** The navigation cost of the edge */
			double weight;

			/** Goal Reaching Utility GRU on this edge 
			 *
			 * To use in a directed graph. Each edge holds the GRU information
			 * of the node it is leading to.
			 */
			double gru;
			
			/** The cluster the edge belongs to
			 *
			 * This is useful to track all information about that edge with
			 * respect to its situation in the quadtree decomposition this in
			 * the geographical data
			 */
			RasterCellIndex cluster;

			NavGEdge& operator=(const NavGEdge &rhs);

		};

	}
}

#endif /* LGL_NAVIGATION_GRAPH_EDGE_HPP */

