/**
 * \file PositionMap.cpp
 *
 * Header file declaring the position map and related structures
 * 
 *
 * \date 2009-11-18
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_POSITION_MAP_HPP
#define LGL_POSITION_MAP_HPP

#include <vector>
#include <ostream>

#include <lgl/FlexGrid.hpp>
#include <lgl/GeoData.hpp>

namespace jafar {
  namespace lgl {

	 /** TODO comments on Structure Pos2d */
	 struct Pos2d {
		int x; 
		int y;

		Pos2d():x(0),y(0) {}
		Pos2d(int _x,int _y):x(_x),y(_y) {}
  
		void setPos(int _x, int _y) {x=_x;y=_y;}


		bool operator==(const  Pos2d &rhs) const
		{
		  return (x == rhs.x) && (y == rhs.y);
		}  

	 };

	 /** TODO comments on Structure  RasterIndex2d */
	 struct  RasterIndex2d {
  
		int i; 
		int j;

		RasterIndex2d():i(0),j(0) {}
		RasterIndex2d(int _i,int _j):i(_i),j(_j) {}
  
		void setPos(int _i, int _j) {i=_i;j=_j;}
  
	 };

	 /** Definition of output operator for Pos2d structure */
	 inline std::ostream& operator<< (std::ostream& out, const Pos2d& pos) {
		out << "(" << pos.x <<":" << pos.y << ")";
		return out;
	 }
		
	 /** Definition of output operator for RasterIndex2d structure */
	 inline std::ostream& operator<< (std::ostream& out, const RasterIndex2d& cell) {
		out << "(" << cell.i <<":" << cell.j << ")";
		return out;
	 }

	 enum {LEFT, TOP, RIGHT, BOTTOM};
	 
	 typedef FlexGrid<Pos2d> PosMap;

	 /**
	  * The geometric decomposition (quadtree, bintree or any rectangular-pattern based decomposition) 
	  * of the terrain represented over the position map
	  */
	 class PositionMap {
	  
	 private :

		int rasterxsize, rasterysize;

		// this array contains all LEFT/RIGHT positions for raster cells
		PosMap xpos;
		// this array contains all TOP/BOTTOM positions for raster cells
		PosMap ypos;
	  
		friend class NavGraph;

	 public:
	  
		PositionMap(const GeoData& grid);
			
		~PositionMap() {}
			
		bool getPosForRasterCell(int i, int j, std::vector<Pos2d>& pos) const;
		int getRasterCell(std::vector< RasterIndex2d>& cells, const Pos2d& pos) const;
			
		// get all the neighboors of the given position
		bool getNeighboors(const Pos2d& pos, std::vector<Pos2d*>& nb);
			
		int nbPositions() const {
		  return xpos.size()+ypos.size();
		}

	 };
  }
}

#endif //
