/**
 * \file PursuitTaskManager.hpp
 *
 * Header file for the pursuit task manager class
 * 
 * \author Cyril Robin
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_PURSUIT_TASK_MANAGER_HPP
#define LGL_PURSUIT_TASK_MANAGER_HPP

#include <lgl/PursuitGraph.hpp>
#include <lgl/CellID.hpp>

//#include <boost/weak_ptr.hpp>

//#include <vector>

#ifndef PURSUIT_TASK_ID
#define PURSUIT_TASK_ID

typedef long int PursuitTask_ID ;

#endif

namespace jafar {
	namespace lgl {

		/**
		 * PursuitTask is the class used to describe a support task during the pursuit mission.
		 */
		class PursuitTask {
 		///{{{ 
		

			public : 
				// variables
				unsigned int 	time ;				// when
				Cell_ID 		place ;				// where
				int				asking_robot;		// ID of the robot who created the task
				PursuitTask_ID	id ;				// ID of the task
				// priority level ?
				int				robot_in_charge;	// ID of the robot in charge of the task


				/** constructor */
				PursuitTask() {}

				PursuitTask(
								unsigned int 	_time,
								Cell_ID 		_place,
								int				_asking_robot,
								PursuitTask_ID	_id,
								int				_robot_in_charge = -1
					) :
						time (_time),
						place(_place),
						asking_robot(_asking_robot),
						id(_id),
						robot_in_charge( _robot_in_charge)
					{ }
			
				/** Destructor */	
				~PursuitTask() { }

				/** operator < 
				 * /!\ does not take into account all the parameters, but only the time and place...
				 */
				bool operator< (const PursuitTask &rhs) const ; 

				/** operator == 
				 * /!\ does not take into account the parameter "robot_in_charge" nor the ID...
				 */
				bool operator== (const PursuitTask &rhs) const ; 

		};///}}}

		/** structure for sorting the Pursuit Tasks */
		struct PursuitTaskKey 
		{///{{{
			unsigned int 	time ;				// when
			Cell_ID 		place ;				// where

			bool operator< (const PursuitTaskKey &rhs) const
			{
				if ( time != rhs.time ) return ( time < rhs.time );
				else					return ( place < rhs.place );
			}

		};///}}}


		/**
		 * PursuitTaskManager is the class used to manage the support task during the pursuit mission.
		 */
		class PursuitTaskManager {
 		///{{{ 
		
			private :
				int robotID ;			// ID of the owner

				PursuitTask_ID countTaskID ;	// counter for the pursuit task IDs 

				std::map< PursuitTaskKey, PursuitTask > oldTaskList ;
				std::map< PursuitTaskKey, PursuitTask > newTaskList ;

				std::vector< PursuitTask_ID > obsoleteTasks ;		// the tasks that are not needed anymore
				std::vector< PursuitTask_ID > nonattributedTasks ;	// the tasks for no robot is in charge with 

			public : 

				/** constructor */
				//PursuitTaskManager() { }
				PursuitTaskManager( int _robotID = -1 ) ;
			
				/** Destructor */	
				~PursuitTaskManager() { }

				/*  Actions */
				void updateTaskList( const PursuitGraph *aPG) ;

		};///}}}

	}
}

#endif /* LGL_PURSUIT_TASK_MANAGER_HPP */
