/**
 * \file Stats.hpp
 *
 * Header file for the statistics class
 * 
 *
 * \author Cyril Robin
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_STATS_HPP
#define LGL_STATS_HPP

#include <ostream>

namespace jafar {
	namespace lgl {
		
		template < class Class >
		class Stats {
 		///{{{ 
		
			private :
				// nbr of encountered variable
				unsigned int count ;

				// sum of all the previous values
				Class sum ;	

				// mean
				Class mean ;

				// min
				Class min ;
				
				// max
				Class max ;

			public : 
				/** Constructor & Destructor */
				Stats () : count(0) 
				{ }

				~Stats() 
				{ }
			
				/**
				 * update the statistics according to the new value.
				 *
				 * @param newValue	a new encountered Value 
				 */
				void update ( Class const &newValue )
				{///{{{
					count ++;

					if ( count == 1 )
					{
						sum 	= newValue ;
						mean	= newValue ;
						min 	= newValue ;
						max 	= newValue ;
					}
					else
					{
						sum    += newValue ;
						mean	= Class ( sum / count ) ;
						min 	=  ( min > newValue ? newValue : min ) ;
						max 	=  ( max < newValue ? newValue : max ) ;
					}

				}///}}}
				 
				/**
				 * reset the statistics.
				 */
				void reset()
				{///{{{
					count = 0 ;
				}///}}}

				/** Get functions
				 */
				Class getcount() 	const 	{ return(count); 	}
				Class getsum() 		const 	{ return(sum);	}
				Class getmean()		const 	{ return(mean);	}
				Class getmin()		const 	{ return(min);	}
				Class getmax()		const 	{ return(max);	} 
				
		};///}}}

		/* output stream for PursuitGNode */
		template < class Class >
		std::ostream& operator<<(std::ostream& out, Stats < Class > const &rhs)
		{///{{{
		
			out << "Stats over " 	<< rhs.getcount() 	<< " values : [ " ;
			if ( rhs.getcount() > 0 )
				out	<< "Sum = " 	<< rhs.getsum()		<< " | "
					<< "Mean = " 	<< rhs.getmean()	<< " | "
					<< "Min = " 	<< rhs.getmin()		<< " | "
					<< "Max = " 	<< rhs.getmax()		<< " ]"	;
			else
				out << " no stats ]" ;

			out << std::endl ;

			return (out);

		}///}}}

	}
}

#endif /* LGL_STATS_HPP */
