/**
 * \file LGLPursuitViewer.cpp
 *
 * Source file that defines a small IHM for the Pursuit Module
 *
 * \author Cyril Robin 
 *
 * \ingroup lgl
 *
 **/

#ifdef HAVE_MODULE_QDISPLAY // lgl viewer is on if HAVE_MODULE_QDISPLAY is defined

#include <lgl/LGLMultiPursuit_Viewer.hpp>
#include <lgl/PursuitGNode.hpp>

#include <exception>
#include <iostream>


#include <QGraphicsScene>
#include <QGraphicsView>

#ifndef SWITCH_CONNEXITY
#define SWITCH_CONNEXITY 0
// use with "findAccessibilxwity()" functions
// 0 is for 9-connexity
// 1 is for 5-connexity ( '+' cross)
#endif

#ifndef SCALE
//#define SCALE 4.0
#define SCALE 8.0
#endif

using namespace jafar::qdisplay;
using namespace std;

namespace jafar {
	namespace lgl {

		/****************************************************************/
		/************************ PURSUIT_CONTROLER *********************/
		/****************************************************************/
		
		void MultiPursuit_Controller::keyPressEvent ( QKeyEvent * event )
		{///{{{
			
			switch (event->key()) {

				// Control of the Target {{{
				#if SWITCH_CONNEXITY

				case Qt::Key_Return:
					std::cout<<"-> You stay here !"<<std::endl;
					master->move( +0, +0 );
					break;
				case Qt::Key_Left:
					std::cout<<"-> You go left !"<<std::endl;
					master->move( -1, +0 );
					break;
				case Qt::Key_Right:
					std::cout<<"-> You go right !"<<std::endl;
					master->move( +1, +0 );
					break;
				case Qt::Key_Down:
					std::cout<<"-> You go down !"<<std::endl;
					master->move( +0, +1 );
					break;
				case Qt::Key_Up:
					std::cout<<"-> You go up !"<<std::endl;
					master->move( +0, -1 );
					break;

				#else 

				case Qt::Key_S:
					std::cout<<"-> You stay here !"<<std::endl;
					master->move( +0, +0 );
					break;
				case Qt::Key_Q:
					std::cout<<"-> You go left !"<<std::endl;
					master->move( -1, +0 );
					break;
				case Qt::Key_D:
					std::cout<<"-> You go right !"<<std::endl;
					master->move( +1, +0 );
					break;
				case Qt::Key_X:
					std::cout<<"-> You go down !"<<std::endl;
					master->move( +0, +1 );
					break;
				case Qt::Key_Z:
					std::cout<<"-> You go up !"<<std::endl;
					master->move( +0, -1 );
					break;
				case Qt::Key_A:
					std::cout<<"-> You go up and left !"<<std::endl;
					master->move( -1, -1 );
					break;
				case Qt::Key_E:
					std::cout<<"-> You go up and right !"<<std::endl;
					master->move( +1, -1 );
					break;
				case Qt::Key_W:
					std::cout<<"-> You go down and left !"<<std::endl;
					master->move( -1, +1 );
					break;
				case Qt::Key_C:
					std::cout<<"-> You go down and right !"<<std::endl;
					master->move( +1, +1 );
					break;
				#endif
				///}}}

				// Option {{{
				case Qt::Key_H:
					std::cout <<"-> Displaying help : " << std::endl ;
					master->displayhelp() ;
					break;
				case Qt::Key_I:
					std::cout <<"-> Displaying Statistics : " << std::endl ;
					master->displaystats() ;
					break;
				case Qt::Key_Space:
					if ( ! master->visibleRisks) 
					{
						std::cout<<"-> Displaying the risk positions..."<<std::endl;
						master->displayrisks();
					} else {
						std::cout<<"-> Hiding the risk positions..."<<std::endl;
						master->hiderisks();
					}
					break;
				case Qt::Key_Escape:
					std::cout<<"-> Exiting the game... :-("<<std::endl;
					std::cout<<"Good bye! See you soon!"<<std::endl;
					exit(0);
					break;
				default :
					std::cout<<"-> Not a valid command !!!"<<std::endl;
				///}}}
			}

			std::cout << std::endl << ">>> Next Action ? " << std::flush ;

		}///}}}

		void MultiPursuit_Controller::setMaster( LGLMultiPursuit_Viewer &_master)
		{///{{{
			master = &_master;
		}///}}}

		/****************************************************************/
		/************************ PURSUIT_VIEWER ************************/
		/****************************************************************/

		void MultiPursuit_Viewer::mouseMoveEvent ( QMouseEvent * event )
		{///{{{

			QGraphicsView::mouseMoveEvent(event);

			setTitleWithMouseCoordinates( event->x() * 2.0 / scale, event->y() * 2.0 / scale );

		}///}}}

		/****************************************************************/
		/*********************** LGL_PURSUIT_VIEWER *********************/
		/****************************************************************/

		// Constructor
		LGLMultiPursuit_Viewer::LGLMultiPursuit_Viewer(RobotPursuer &_pt_robot):
			pt_robot (&_pt_robot)
		{ //{{{
		
			scale = SCALE ;

			// viewers with screen coordinates (corner 0,0 is at the top left, x is the horizontal axis)
			viewer1 = new MultiPursuit_Viewer(scale);
			viewer2 = new MultiPursuit_Viewer(scale);
			viewer3 = new MultiPursuit_Viewer(scale);
			
			viewer1->setTitle("Robot Accessibility");
			viewer2->setTitle("Target Accessibility");
			viewer3->setTitle("Robot Visibility");

			// controller
			ctrl 	= new jafar::lgl::MultiPursuit_Controller();
			ctrl->setMaster(*this);
			ctrl->grabKeyboard();

			ctrl->setTitle("Controller");

			// robot initialisation
			setRobotPosition(-1.0 * scale,-1.0*scale);

			robot.shape1 = new Shape(Shape::ShapeRectangleFilled, robot.x * scale /2, robot.y * scale /2, 1.0 * scale, 1.0 * scale);
			robot.shape2 = new Shape(Shape::ShapeRectangleFilled, robot.x * scale /2, robot.y * scale /2, 1.0 * scale, 1.0 * scale);
			robot.shape3 = new Shape(Shape::ShapeRectangleFilled, robot.x * scale /2, robot.y * scale /2, 1.0 * scale, 1.0 * scale);

			robot.shape1->setColor(0,0,250);
			robot.shape2->setColor(0,0,250);
			robot.shape3->setColor(0,250,0);

			robot.shape1->setFontSize( 1 * scale);
			robot.shape2->setFontSize( 1 * scale);
			robot.shape3->setFontSize( 1 * scale);

			robot.shape1->setLabel("R1");
			robot.shape2->setLabel("R1");
			robot.shape3->setLabel("R1");

			robot.shape1->setFontColor(0,0,250);
			robot.shape2->setFontColor(0,0,250);
			robot.shape3->setFontColor(0,250,0);


			// target initialisation
			setTargetPosition(-1.0 * scale,-1.0*scale);

			target.shape1 = new Shape(Shape::ShapeEllipseFilled, target.x * scale /2, target.y * scale /2, 1.0 * scale, 1.0 * scale);
			target.shape2 = new Shape(Shape::ShapeEllipseFilled, target.x * scale /2, target.y * scale /2, 1.0 * scale, 1.0 * scale);
			target.shape3 = new Shape(Shape::ShapeEllipseFilled, target.x * scale /2, target.y * scale /2, 1.0 * scale, 1.0 * scale);

			target.shape1->setColor(0,0,100);
			target.shape2->setColor(0,0,100);
			target.shape3->setColor(250,250,250);

			target.shape1->setFontSize( 1 * scale);
			target.shape2->setFontSize( 1 * scale);
			target.shape3->setFontSize( 1 * scale);

			target.shape1->setLabel("T");
			target.shape2->setLabel("T");
			target.shape3->setLabel("T");

			target.shape1->setFontColor(0,0,100);
			target.shape2->setFontColor(0,0,100);
			target.shape3->setFontColor(250,250,250);

			//risk
			visibleRisks = false ;

		} //}}}

		LGLMultiPursuit_Viewer::~LGLMultiPursuit_Viewer()
		{///{{{
			delete viewer1;
			delete viewer2;
			delete viewer3;

			delete ctrl;

			for (unsigned int i = 0 ; i < risks.size() ; i++ ) 
				delete risks.at(i);

		}///}}}
	
		bool LGLMultiPursuit_Viewer::start() 
		{///{{{

			std::cout << "   ********************************* [LGL_MULTI_PURSUIT_VIEWER] ********************************   " << std::endl ;

			bool b = true;

			timeval t;
			double d;

			gettimeofday(&t,NULL);
			d = 1000.0 * t.tv_sec + t.tv_usec/ 1000.0;

			if (pt_robot == NULL)
			{
				std::cerr << "No robot specified. Cannot run the game." << std::endl;
				return(!b);
			}
			
			// start the robots
			//pt_robot->start(target.x, target.y, robot.x, robot.y);
			boost::thread th ( boost::bind( &RobotPursuer::start, pt_robot, target.x, target.y) );
			th.join();
			pt_robot->getPosition(robot.x,robot.y);

			// Information
			std::cout << "#---III---  The robot  stands in (" << robot.x  << "," << robot.y  << ")." << std::endl;
			std::cout << "#---III---  The target stands in (" << target.x << "," << target.y << ")." << std::endl;

			// Initialization of the display
			b = startDisplay() ;
			if (!b) return(b);

			// Update the display according to the data
			refresh();

			gettimeofday(&t,NULL);
			d = 1000.0 * t.tv_sec + t.tv_usec/ 1000.0 - d;
			
			std::cout << "#---III---  Game initialized in " << d << "ms." << std::endl;

			std::cout << "#---III---  [ temporal horizon is "<< pt_robot->getPG()->gethorizon() 
								<< " with front = "<< pt_robot->getPG()->getfront().size() 
								<< " - risks = " << pt_robot->getPG()->getrisk().size() 
								<<" - danger = " << pt_robot->getPG()->getdanger().size() 
								<< " ] " << std::endl ; 

			std::cout << std::endl;

			// statistics
			stats.update(pt_robot->getPG(), double ( d / pt_robot->getPG()->gethorizon() ) );

			std::cout << "   ********************************* [LGL_MULTI_PURSUIT_VIEWER] ********************************   " << std::endl ;

			// Start the game
			std::cout << "   /!\\ Now, it's up to you : use keyboard to control the target ! (Press \"H\" Key for some help )" << std::endl ;

			std::cout << std::endl << ">>> Next Action ? " << std::flush ;

			return( b );

		}///}}}

		bool LGLMultiPursuit_Viewer::startDisplay() 
		{///{{{
			bool b = true ;
			
			// robot accessibility 
			try {
				ImageView* imgV1 = new ImageView(pt_robot->getRobotAccessibilityMap());
				imgV1->setPos( 0 , 0 );	// top-left
				imgV1->setScale(scale);
				viewer1->setImageView(imgV1);
			} 
			catch (const std::exception& e)
			{
				b = false ;
				std::cerr << "Couldn't add robot accessibility on the pursuit viewer." << std::endl;
				//throw;
			}

			// target accessibility 
			try {
				ImageView* imgV2 = new ImageView(pt_robot->getTargetAccessibilityMap());
				imgV2->setPos( 0 , 0 );	// top-right
				imgV2->setScale(scale);
				viewer2->setImageView(imgV2);
			} 
			catch (const std::exception& e)
			{
				b = false ;
				std::cerr << "Couldn't add target accessibility on the pursuit viewer." << std::endl;
				//throw;
			}
			
			// robot visibility 
			try {
				ImageView* imgV3 = new ImageView(pt_robot->getRobotVisibilityMap());
				imgV3->setPos( 1 , 1 );	// bottow-left
				imgV3->setScale(scale);
				viewer3->setImageView(imgV3);
			} 
			catch (const std::exception& e)
			{
				b = false ;
				std::cerr << "Couldn't add target accessibility on the pursuit viewer." << std::endl;
				//throw;
			}

			viewer1->setGeometry( 0, 0, pt_robot->getPG()->getxsize() *  scale/2, pt_robot->getPG()->getysize() * scale/2 );
			viewer2->setGeometry( 0, pt_robot->getPG()->getysize() * scale/2 + 70, pt_robot->getPG()->getxsize() *  scale/2, pt_robot->getPG()->getysize() * scale/2 );
			viewer3->setGeometry( pt_robot->getPG()->getxsize() * scale/2 + 20,	0, pt_robot->getPG()->getxsize() *  scale/2, pt_robot->getPG()->getysize() * scale/2 );

			ctrl->setGeometry( pt_robot->getPG()->getxsize() * scale/2 + 20, pt_robot->getPG()->getysize() * scale/2 + 70, 200, 100);

			// Display the robot position
			robot.shape1->setPos(robot.x, robot.y);
			robot.shape2->setPos(robot.x, robot.y);
			robot.shape3->setPos(robot.x, robot.y);

			target.shape1->setPos(target.x, target.y);
			target.shape2->setPos(target.x, target.y);
			target.shape3->setPos(target.x, target.y);

			viewer1->addShape(robot.shape1);
			viewer2->addShape(robot.shape2);
			viewer3->addShape(robot.shape3);

			viewer1->addShape(target.shape1);
			viewer2->addShape(target.shape2);
			viewer3->addShape(target.shape3);

			return (b);

		}///}}}

		bool LGLMultiPursuit_Viewer::refresh()
		{///{{{{

			// Controler
			switch( pt_robot->getPG()->getroot()->risk )
			{
				case OK 					: 	ctrl->setBackgroundColor(   0, 255,  0 ); break;
				case RISK 					: 	ctrl->setBackgroundColor( 255, 255,  0 ); break;
				case DANGER_HIDDEN 			: 	ctrl->setBackgroundColor( 255,   0,  0 ); break;
				case DANGER_LOST 			: 	ctrl->setBackgroundColor( 255,   0,  0 ); break;
				case DANGER_UNSEEN 			: 	ctrl->setBackgroundColor( 255,   0,  0 ); break;
				case DANGER_DISAPPEARED 	: 	ctrl->setBackgroundColor(   0,   0,  0 ); break;

				default 	: 	ctrl->setBackgroundColor(  255, 255, 255); 
			}
				
			// Update risks and danger
			for (unsigned int i = 0 ; i < risks.size() ; i++ ) 
				delete risks.at(i);
			risks.clear();
			
			std::set< Cell_ID>::const_iterator itS ;
			int ysize = pt_robot->getPG()->getysize();

			//danger
			///{{{
			std::set<Cell_ID> _danger;
			for (unsigned int i = 0 ; i < pt_robot->getPG()->getdanger().size() ; i++ ) 
			{
				boost::shared_ptr< PursuitGNode> ptr = (pt_robot->getPG()->getdanger().at(i)).lock() ;
				if (ptr)
					_danger.insert(ptr->targetPos.begin(), ptr->targetPos.end() );
			}

			for (itS = _danger.begin() ; itS != _danger.end() ; ++ itS)
			{
				for ( int j=0 ; j<3; j++)
				{
					//risks.push_back(new Shape( Shape::ShapeEllipseFilled, (*itS / ysize)*scale/2, (*itS % ysize)*scale/2, 0.5*scale, 0.5*scale, 0) );
					risks.push_back(new Shape( Shape::ShapeCross, (*itS / ysize)*scale/2, (*itS % ysize)*scale/2, 0.6*scale, 0.6*scale, 45) );
					risks.at(risks.size()-1)->setColor( 150, 150,  50 );
					if (!visibleRisks)	
						risks.at(risks.size()-1)->setVisible(false);
				}
				viewer1->addShape(risks.at(risks.size()-3));
				viewer2->addShape(risks.at(risks.size()-2));
				viewer3->addShape(risks.at(risks.size()-1));
			}
			///}}}

			//risk
			///{{{
			std::set<Cell_ID> _risk;
			for (unsigned int i = 0 ; i < pt_robot->getPG()->getrisk().size() ; i++ ) 
			{
				boost::shared_ptr< PursuitGNode> ptr = (pt_robot->getPG()->getrisk().at(i)).lock() ;
				if (ptr)
					_risk.insert(ptr->targetPos.begin(), ptr->targetPos.end() );
			}

			for (itS = _risk.begin() ; itS != _risk.end() ; ++ itS)
			{
				for ( int j=0 ; j<3; j++)
				{
					//risks.push_back(new Shape( Shape::ShapeRectangleFilled, (*itS / ysize)*scale/2, (*itS % ysize)*scale/2, 0.3*scale, 0.3*scale, 0) );
					risks.push_back(new Shape( Shape::ShapeCross, (*itS / ysize)*scale/2, (*itS % ysize)*scale/2, 0.2*scale, 0.2*scale, 0) );
					risks.at(risks.size()-1)->setColor( 100, 100, 100 );
					if (!visibleRisks)	
						risks.at(risks.size()-1)->setVisible(false);
				}
				viewer1->addShape(risks.at(risks.size()-3));
				viewer2->addShape(risks.at(risks.size()-2));
				viewer3->addShape(risks.at(risks.size()-1));
			}
			///}}}

			// Move the robot position
			robot.shape1->setPos(robot.x * scale /2, robot.y * scale /2);
			robot.shape2->setPos(robot.x * scale /2, robot.y * scale /2);
			robot.shape3->setPos(robot.x * scale /2, robot.y * scale /2);

			// Move the target position
			target.shape1->setPos(target.x * scale /2, target.y * scale /2);
			target.shape2->setPos(target.x * scale /2, target.y * scale /2);
			target.shape3->setPos(target.x * scale /2, target.y * scale /2);

			return (true);
		}///}}}

		void LGLMultiPursuit_Viewer::displayrisks()
		{///{{{
			visibleRisks = true ;
			for (unsigned int i=0 ; i< risks.size() ; i++)
				risks.at(i)->setVisible(true);
		}///}}}

		void LGLMultiPursuit_Viewer::hiderisks()
		{///{{{
			visibleRisks = false ;
			for (unsigned int i=0 ; i< risks.size() ; i++)
				risks.at(i)->setVisible(false);
		}///}}}

		void LGLMultiPursuit_Viewer::setRobotPosition( int _x, int _y)
		{///{{{
			robot.x = _x;
			robot.y = _y;
		}///}}}

		void LGLMultiPursuit_Viewer::setTargetPosition( int _x, int _y)
		{///{{{
			target.x = _x;
			target.y = _y;
		}///}}}

		void LGLMultiPursuit_Viewer::move( int _x, int _y)
		{///{{{


			timeval t;
			double d, dd;
			
			// Check validity of the movement
			int newTargetPosx, newTargetPosy;
			
			newTargetPosx = target.x + 2 * _x ;
			newTargetPosy = target.y + 2 * _y ;

			newTargetPosx = newTargetPosx < 0 ? 0 : newTargetPosx ;
			newTargetPosy = newTargetPosy < 0 ? 0 : newTargetPosy ;

			newTargetPosx = (unsigned int) newTargetPosx >= pt_robot->getPG()->getxsize() ? pt_robot->getPG()->getxsize()-2 : (newTargetPosx/2) * 2 + 1 ;
			newTargetPosy = (unsigned int) newTargetPosy >= pt_robot->getPG()->getysize() ? pt_robot->getPG()->getysize()-2 : (newTargetPosy/2) * 2 + 1 ;

			// check with geodata
			if ( pt_robot->getPG()->getnavigator()->gettarGeoData()->getValueFromBand( GEODATA_TFLAT_BAND, newTargetPosx/2, newTargetPosy/2 ) > 0.5 ) 
			{
	
				// Updating target position
				target.x = newTargetPosx ;
				target.y = newTargetPosy ;

				// running for robot
				gettimeofday(&t,NULL);
				d = 1000.0 * t.tv_sec + t.tv_usec/ 1000.0;

				//pt_robot->run(target.x, target.y, robot.x, robot.y);
				boost::thread th ( boost::bind( &RobotPursuer::run, pt_robot, target.x, target.y) );
				th.join();
				pt_robot->getPosition(robot.x, robot.y);

				// Refreshing the display
				gettimeofday(&t,NULL); dd = 1000.0 * t.tv_sec + t.tv_usec/ 1000.0;
					refresh();

				gettimeofday(&t,NULL); dd = 1000.0 * t.tv_sec + t.tv_usec/ 1000.0 - dd;
					
				d = 1000.0 * t.tv_sec + t.tv_usec/ 1000.0 - d; 

				// output	
				std::cout 	<< "You move to (" << target.x << "," << target.y << ") and " ; 

				switch ( pt_robot->getPG()->getroot()->risk )
				{
					case OK	:
						std::cout << "the robot follows you in (" << robot.x << "," << robot.y << ") and is watching you... "; break;
					case RISK :
						std::cout << "the robot follows you in (" << robot.x << "," << robot.y << ") but cannot see you ! "; break ;
					case DANGER_HIDDEN :	
						std::cout << "the robot failed ! (you hide ;-) ) "; break;
					case DANGER_LOST :
						std::cout << "the robot failed ! (cannot trace you anymore :P ) "; break;
					case DANGER_UNSEEN : 	
						std::cout << "the robot failed ! (hasn't seen you for too much time...) "; break;
					case DANGER_DISAPPEARED : 	
						std::cout << "the robot failed ! (you just disappeared ! oO ) "; break;
					default :
							;//throw exception ?
				}

				std::cout 	<< "[ temporal horizon is "<< pt_robot->getPG()->gethorizon() 
								<< " with front = "<< pt_robot->getPG()->getfront().size() 
								<< " - risks = " << pt_robot->getPG()->getrisk().size() 
								<<" - danger = " << pt_robot->getPG()->getdanger().size() 
							<< " ] " << std::endl ; 
				std::cout 	<< "[computed in " << d << " ms"
								<< " including " 
								<< dd << " ms for display." 
							<< "]" << std::endl;

				// statistics
				stats.update( pt_robot->getPG(), d );


			} else {
				std::cout<< "You are NOT allowed to go there ! (" << newTargetPosx << "," << newTargetPosy << ") - Please choose another action." << std::endl;
			}

		}///}}}

		void LGLMultiPursuit_Viewer::displaystats()
		{///{{{
			std::cout << stats ;
		}///}}}

		void LGLMultiPursuit_Viewer::displayhelp()
		{///{{{
			std::cout << " Use keyboard to control the target !" << std::endl ;
				// control command
				#if SWITCH_CONNEXITY
				std::cout << "\t\t ( Up     \t=>\t Move Up                )" << std::endl ;
				std::cout << "\t\t ( Down   \t=>\t Move Down              )" << std::endl ;
				std::cout << "\t\t ( Left   \t=>\t Move Left              )" << std::endl ;
				std::cout << "\t\t ( Right  \t=>\t Move Right             )" << std::endl ;
				std::cout << "\t\t ( Return \t=>\t Stay here              )" << std::endl ;
				#else
				std::cout << "\t\t ( Z      \t=>\t Move Up                )" << std::endl ;
				std::cout << "\t\t ( X      \t=>\t Move Down              )" << std::endl ;
				std::cout << "\t\t ( Q      \t=>\t Move Left              )" << std::endl ;
				std::cout << "\t\t ( D      \t=>\t Move Right             )" << std::endl ;
				std::cout << "\t\t ( S      \t=>\t Stay here              )" << std::endl ;
				std::cout << "\t\t ( A      \t=>\t Move Up and Left       )" << std::endl ;
				std::cout << "\t\t ( E      \t=>\t Move Up and Right      )" << std::endl ;
				std::cout << "\t\t ( W      \t=>\t Move Down and Left     )" << std::endl ;
				std::cout << "\t\t ( C      \t=>\t Move Down and Right    )" << std::endl ;
				#endif

				std::cout << std::endl ;

				// option command
				std::cout << "\t\t ( H  	\t=>\t Display help           )" << std::endl ;
				std::cout << "\t\t ( I  	\t=>\t Display statistics     )" << std::endl ;
				std::cout << "\t\t ( Space  \t=>\t Display/Hide the risks )" << std::endl ;
				std::cout << "\t\t ( Escape \t=>\t Exit                   )" << std::endl ;
				
		}///}}}

	}
}

#endif
