/**
 * \file PositionMap.cpp
 *
 * 
 *
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/


#include <lgl/PositionMap.hpp>

namespace jafar {
  namespace lgl {

	 PositionMap::PositionMap(const GeoData& grid) {
		rasterxsize=grid.dataset->GetRasterXSize();
		rasterysize=grid.dataset->GetRasterYSize();

		xpos.resize(0,rasterxsize,0,rasterysize+1,Pos2d());
		ypos.resize(0,rasterxsize+1,0,rasterysize,Pos2d());

		// init x pos
		int i,j;
		for (i=0;i<rasterxsize;i++) {
		  for (j=0;j<(rasterysize+1);j++) {
			 // compute the position element (x,y) inside the xpos map at (i,j)
			 // x = i*2+1
			 // y = y*2
			 xpos.at(i,j).setPos((i*2)+1,j*2);
		  }
		}

		// init y pos
		for (i=0;i<(rasterxsize+1);i++) {
		  for (j=0;j<rasterysize;j++) {
			 // x = 2*i
			 // y = j*2+1
			 ypos.at(i,j).setPos(i*2,(j*2)+1);
		  }
		}

	 }


	 bool PositionMap::getPosForRasterCell(int i,int j, std::vector<Pos2d>& pos) const {
		pos.clear();

		if ((i<0)||(i>=rasterxsize)||(j>=rasterysize)||(j<0)) {
		  return false; 
		}

		// get 2 x positions
		// LEFT
		pos.push_back(xpos.at(i,j));
		// RIGHT
		pos.push_back(xpos.at(i,j+1));

		// get 2 y positions
		// TOP
		pos.push_back(ypos.at(i,j));
		// BOTTOM
		pos.push_back(ypos.at(i+1,j));

		return true;
	 }

	 /**
	  * Get the raster cells related to this position
	  */
	 int PositionMap::getRasterCell(std::vector<RasterIndex2d>& cells, const Pos2d& pos) const {/*{{{*/
		cells.clear();

		// compute the (i,j) index of the pos element inside the xpos or ypos maps 

		// x position
		if ((pos.x%2) && (pos.y%2==0))  {
		  // LEFT cell
		  if (pos.y>0) {
			 cells.push_back(RasterIndex2d((pos.x-1)/2,(pos.y/2)-1));
		  }
		  // RIGHT cell
		  if (pos.y<rasterysize*2) {
			 cells.push_back(RasterIndex2d((pos.x-1)/2,pos.y/2));
		  }  

		} else {
		  if ((pos.x%2==0) && (pos.y%2)) {

			 // TOP cell
			 if (pos.x>0) {
				cells.push_back(RasterIndex2d((pos.x/2)-1,(pos.y-1)/2));
			 }

			 // BOTTOM cell
			 if (pos.x<(rasterxsize*2)) {
				cells.push_back(RasterIndex2d(pos.x/2,(pos.y-1)/2));
			 }  

		  } else {
			 return cells.size();
		  }	 
		}

		std::cout << cells[0] << std::endl;
		if (cells.size()==2)
		  std::cout << cells[1] << std::endl;

		return cells.size();
	 }/*}}}*/

	 bool PositionMap::getNeighboors(const Pos2d& pos, std::vector<Pos2d*>& nb) {
		return true;
	 }

  }
}
