/**
 * \file PursuitGNode.cpp
 *
 * Source file for the pursuit graph node class
 * 
 *
 * \author Cyril Robin
 *
 * \ingroup lgl
 *
 **/


#include <lgl/PursuitGNode.hpp>

#include <ostream> 
#include <iostream> 

namespace jafar {
	namespace lgl {
		
		/****************************************************************/
		/************************ PURSUIT_G_NODE ************************/
		/****************************************************************/

		
		void PursuitGNode::setChildren ( const std::vector< boost::shared_ptr<PursuitGNode> > &newChildren ) 
		{///{{{
				children.clear() ;
				children.insert( children.end(), newChildren.begin(), newChildren.end() );
		}///}}}

		void PursuitGNode::noMoreChildren ()
		{///{{{
			children.clear();
		}///}}}

		void PursuitGNode::addChild ( const boost::shared_ptr<PursuitGNode> &newChild ) 
		{///{{{
			children.push_back( newChild );
		}///}}}

		void PursuitGNode::addChild ( const boost::weak_ptr<PursuitGNode> &newChild ) 
		{///{{{
			children.push_back( newChild );
		}///}}}

		const std::vector< boost::weak_ptr<PursuitGNode> > & PursuitGNode::getChildren () const 
		{///{{{
			return ( children);
		}///}}}

		bool PursuitGNode::operator== (const PursuitGNode &rhs) const
		{///{{{

			//same risk and (both robot and target) positions
			if ( robotPos == rhs.robotPos && risk == rhs.risk && targetPos == rhs.targetPos ) return true;
			
			return false ;

		}///}}}
		
		bool PursuitGNode::equal (const PursuitGNode &rhs) const
		{///{{{

			//same risk and (both robot and target) positions
			// AND same time
			if ( robotPos == rhs.robotPos && risk == rhs.risk && targetPos == rhs.targetPos && time == rhs.time ) return true;
			
			return false ;

		}///}}}
		

		/****************************************************************/
		/********************** / PURSUIT_G_NODE ************************/
		/****************************************************************/

		bool before ( const boost::weak_ptr <PursuitGNode> &lhs, const boost::weak_ptr<PursuitGNode> &rhs) 
		{///{{{ 
			boost::shared_ptr<PursuitGNode> ptr_lhs (lhs), ptr_rhs(rhs) ;

			if (ptr_lhs && ptr_rhs)
				return( ptr_lhs->time < ptr_rhs->time ); 

			return false;
		}///}}}

		/****************************************************************/
		/************************ OUTPUT STREAM *************************/
		/****************************************************************/

		/* output stream for PursuitGNode */
		std::ostream& operator<<(std::ostream& out, PursuitGNode const &rhs)
		{///{{{
			out << " PursuitGNode : " << std::endl ;
			out << " ### robotPos : " << rhs.robotPos << std::endl;
			out << " ### risk     : " << rhs.risk << std::endl;
			out << " ### targetPos: " ;
			for (; ;)
			{

			}
			out << "### children : [ # " << rhs.getChildren().size() << " ] " << std::endl ;

			return (out) ;
			
		}///}}}

	}
}

